﻿from tkinter import *
#import time

#■配列宣言
#
#  N1,N2  : 配列の大きさ
#  戻り値 : ゼロ設定された配列(A[N1][N2]を宣言したこととなる）
#           実装的には[N2個のリスト，N2個のリスト，・・・]となる。
#
def array2(N1,N2):
    return [[0 for j in range(N2)]for k in range(N1)]

#■画素初期設定(画素をピクセル1の正方形で代表させる)
#
#  canvas : Canvasクラスオブジェクト
#  NX     : X方向最大数+1を指定すること
#  NY     : Y方向最大数+1を指定すること
#  戻り値 : 画素IDの配列 
#
def initDt(canvas, NX,NY):
    A=array2(NX,NY)
    for i in range(NX):
        for j in range(NY):
            A[i][j]=canvas.create_rectangle(i,j,i,j,fill='blue', outline='')   
    return A

#■Tk初期設定
#
#  Title  : 最初に表示するウィンドウのタイトル
#  戻り値 : Tkクラスオブジェクト 
#
def initTk(Title):
    tk=Tk()
    tk.title("Mandelbrot")
    tk.resizable(0,0)
    return tk

#■キャンバス初期設定
#
#  tk : Tkクラスオブジェクト
#  W  : キャンバスの幅
#  H  : キャンバスの高さ
#  戻り値 : Canvasクラスオブジェクト
#
def initCanvas(tk, W,H):
    canvas=Canvas(tk,width=W,height=H,highlightthickness=0)
    canvas.pack()
    return canvas
 
#■繰返し演算
#
#  C    　:  漸化式　Zn = Zn^2+C のC
#  LOOP 　:  収れん判定のための繰返し回数
#  戻り値 :  収れん回数/発散検出回数
#　　　　　　0 のとき，LOOP回数では収れん/発散を判定できなかったことを示す。
#
def Iter(C, LOOP):
    Z=complex(0)
    for i in range(1,LOOP+1):
        # 以下を ZZ=Z*Z*Z+C としても興味深い画像になる
        ZZ=Z*Z+C
        if(abs(ZZ)>=4):
            return -i
        if(abs(ZZ-Z)<0.001):
            return i
        Z=ZZ
    return 0

#■マンデルブロー集合描画
#
#  canvas  : 描画用キャンバス
#  CID[][] : 描画用の点識別番号配列
#　DX      : 点の刻み幅
#  X0      : 実数部の計算開始位置
#　Y0      : 虚数部の計算開始位置
#  XE      : 実数部の計算終了位置
#  YE      : 虚数部の計算終了位置
#  LOOP    : 計算回数
#  MD      : 表示時のModulus
#  Ctab1   : 正のときの色表示テーブル(MD個の配列)
#  Ctab2   : 負のときの色表示テーブル(MD個の配列)
#
def drawFrac(canvas, CID,DX,X0,Y0, XE,YE, LOOP,MD,Ctab1,Ctab2):
    X=X0
    XMAX=len(CID)
    YMAX=len(CID[0])
    while X<=XE:
        #実行途上表示
        if int(X/DX)%5==0:
            tk.update()
        Y=Y0
        while Y<=YE:
            #繰返し回数による色の選定
            k=Iter(complex(X,Y),LOOP)
            if k>=0:
                CL=Ctab1[k % MD]
            else:
                CL=Ctab2[k % MD]
            #表示用座標設定と色変更
            XX=int((X-X0)/DX+0.5)
            YY=int((Y-Y0)/DX+0.5)
            if XX>=0 and XX<XMAX and YY>=0 and YY<YMAX:
                canvas.itemconfig(CID[XX][YY],fill=CL)
            #Y増分
            Y+=DX
        #X増分
        X+=DX
        
#■実行メイン
Ctab1=['black','#000077','#0000FF','#0077FF','#00FFFF']
Ctab2=['#773300','#770000','#FF0000','#FF7700','#FFFF00']

tk=initTk("Mandelbrot")
canvas=initCanvas(tk,300,300)
CID=initDt(canvas,301,301)

tk.title("Mandelbrot集合の描画中")
drawFrac(canvas,CID,0.01,-2,-1.5,1,1.5,200,5, Ctab1,Ctab2)
tk.update()
tk.title("Mandelbrot集合の描画終了")
tk.mainloop()

